/***************************************************************************
 *   Copyright (C) 2008, 2009 by Miguel Escudero Gestal
 *   miguelescudero@users.sourceforge.net
 *   
 *   This file is part of QChartDiary
 *
 *   QChartDiary is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   QChartDiary is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.
 ***************************************************************************/

#include "chart.h"

#include <QDate>
#include <QMessageBox>
#include <QPainter>
#include <QSqlQuery>
#include <QtSql>


//PUBLIC


Chart::Chart(QWidget *parent)
	: QWidget(parent)
{
	chartType = 1;
	tempo = 0;
	setPalette(QPalette(QColor(160, 160, 160)));
	setAutoFillBackground(true);
	lineDaySeparation = 32;
}


//PUBLIC SLOTS


void Chart::calculateTopHour()
{
	QString queryString;
	QSqlQuery query;
	int minuteCount, cycleDaysCount = 0;
	bool last;
	topHour = 60.0;
	
	//Diario
	if(tempo == 0)
	{
		topHour = 24.0;
		return;
	}
	
	for(int i = 0; i < userNames.size(); i++)
	{
		for(int j = 0; j < activeColumnsPerUser[i]; j++)
		{
			queryString = "SELECT * FROM Diary" + userNames[i]
				+ " WHERE Date >= '" + initialDate.toString("yyyy-MM-dd")
				+ "' AND Date <= '" + finalDate.toString("yyyy-MM-dd") + "' ORDER BY Date";
			query.exec(qPrintable(queryString));
			if(!query.next())
				break;
			
			last = false;
			minuteCount = 0;
			cycleDaysCount = 0;
			
			//while(query.next())
			for(int k = 0; k < initialDate.daysTo(finalDate) + 1; k++)
			{
				//Añadimos los nuevos valores si hay registro del día correspondiente
				if(last == false && k == initialDate.daysTo(query.value(1).toDate()))
				{
					minuteCount += query.value(3 + activeColumns[j]).toTime().hour() * 60
						+ query.value(3 + activeColumns[j]).toTime().minute();
					cycleDaysCount++;
				}
				
				switch(tempo)
				{
				case 1: //Semanal
				{
					//Reiniciamos el contador de horas totales al finalizar la semana
					if(initialDate.addDays(k).dayOfWeek() == 7 || k == initialDate.daysTo(finalDate))
					{
						if(minuteCount > topHour)
							topHour = minuteCount;
						minuteCount = 0;
					}
				}
				break;
				case 2: //Media semanal
				{
					//Reiniciamos el contador de horas totales al finalizar la semana
					if(initialDate.addDays(k).dayOfWeek() == 7 || k == initialDate.daysTo(finalDate))
					{
						//Evitamos que se produzca una división por 0
						if(cycleDaysCount == 0)
							cycleDaysCount = 1;
							
						if(minuteCount / cycleDaysCount > topHour)
							topHour = minuteCount / cycleDaysCount;
						minuteCount = 0;
						cycleDaysCount = 0;
					}
				}
				break;
				case 3: //Mensual
				{
					//Reiniciamos el contador de horas totales al finalizar el mes
					if(initialDate.addDays(k).day() == initialDate.addDays(k).daysInMonth()
						|| k == initialDate.daysTo(finalDate))
					{
						if(minuteCount > topHour)
							topHour = minuteCount;
						minuteCount = 0;
					}
				}
				break;
				case 4: //Media mensual
				{
					//Reiniciamos el contador de horas totales al finalizar el mes
					if(initialDate.addDays(k).day() == initialDate.addDays(k).daysInMonth()
						|| k == initialDate.daysTo(finalDate))
					{
						//Evitamos que se produzca una división por 0
						if(cycleDaysCount == 0)
							cycleDaysCount = 1;
							
						if(minuteCount / cycleDaysCount > topHour)
							topHour = minuteCount / cycleDaysCount;
						minuteCount = 0;
						cycleDaysCount = 0;
					}
				}
				break;
				case 5: //Anual
				{
					//Reiniciamos el contador de horas totales al finalizar el año
					if(initialDate.addDays(k).dayOfYear() == initialDate.addDays(k).daysInYear()
						|| k == initialDate.daysTo(finalDate))
					{
						if(minuteCount > topHour)
							topHour = minuteCount;
						minuteCount = 0;
					}
				}
				break;
				case 6: //Media anual
				{
					//Reiniciamos el contador de horas totales al finalizar el año
					if(initialDate.addDays(k).dayOfYear() == initialDate.addDays(k).daysInYear()
						|| k == initialDate.daysTo(finalDate))
					{
						//Evitamos que se produzca una división por 0
						if(cycleDaysCount == 0)
							cycleDaysCount = 1;
							
						if(minuteCount / cycleDaysCount > topHour)
							topHour = minuteCount / cycleDaysCount;
						minuteCount = 0;
						cycleDaysCount = 0;
					}
				}
				break;
				}
				
				//Leemos el siguiente registro o salimos si no hay más registros
				if(last == false && k == initialDate.daysTo(query.value(1).toDate()))
				{
					if(!query.next())
						last = true;
				}
			}
		}
	}
	topHour = topHour / 60.0;
}

void Chart::drawBackgroundLines(QPainter &painter)
{
	//Dibujamos la rejilla
	QPen pen;
	pen.setStyle(Qt::SolidLine);
	pen.setWidth(1);
	pen.setColor(QColor(220, 220, 220));
	painter.setPen(pen);
	
	//Líneas horizontales para cada hora
	for(int i = 0; i <= 24; i++)
	{
		switch(i)
		{
		case 0:
		case 6:
		case 12:
		case 18:
		case 24:
			pen.setWidth(2);
			painter.setPen(pen);
			painter.drawLine(26, 25 + graphicHeight * i / 24, width() -1, 25 + graphicHeight * i / 24);
		break;
		default:
			pen.setWidth(1);
			painter.setPen(pen);
			painter.drawLine(25, 25 + graphicHeight * i / 24, width() - 1, 25 + graphicHeight * i / 24);
		break;
		}
		pen.setWidth(1);
		painter.setPen(pen);
		painter.drawText(1, 29 + graphicHeight * i / 24,
			QString::number(qRound(topHour) - i * qRound(topHour) / 24));
	}
	
	//Línea vertical
	painter.drawLine(25, 25, 25, 25 + graphicHeight);
}

void Chart::drawBars(QPainter &painter)
{
	if(!activeColumns.size())
		return;
		
	painter.setPen(Qt::NoPen);
	
	QString queryString;
	QSqlQuery query;
	int columnsPainted = 0;
	for(int i = 0; i < userNames.size(); i++)
	{
		if(activeColumnsPerUser[i] > 0)
		{
			queryString = "SELECT * FROM Diary" + userNames[i]
				+ " WHERE Date >= '" + initialDate.toString("yyyy-MM-dd")
				+ "' AND Date <= '" + finalDate.toString("yyyy-MM-dd") + "' ORDER BY Date";
			query.exec(qPrintable(queryString));
			while(query.next())
			{
				for(int k = 0; k < activeColumnsPerUser[i]; k++)
				{
					//Asignamos el color
					painter.setBrush(activeColors[k + columnsPainted]);
					
					//Asignamos la altura
					qreal rectHeight =
						- qreal(query.value(3 + activeColumns[k + columnsPainted]).toTime().hour())
						* (graphicHeight / topHour)
						- qreal(query.value(3 + activeColumns[k + columnsPainted]).toTime().minute())
						* (graphicHeight) / (topHour * 60.0);
					
					//Asignamos la posición
					QRectF rect(initialDate.daysTo(query.value(1).toDate()) * 5
						+ initialDate.daysTo(query.value(1).toDate()) * activeColumns.size() * 10
						+ columnsPainted * 10.0 + k * 10.0 + 30.0,
						25.0 + graphicHeight,
						8.0,
						rectHeight);
						
					painter.drawRect(rect);
				}
			}
			columnsPainted += activeColumnsPerUser[i];
		}
	}
	
	//Dibujamos el texto de cada día bajo la gráfica
	QPen pen;
	pen.setStyle(Qt::SolidLine);
	pen.setWidth(1);
	pen.setColor(QColor(220, 220, 220));
	painter.setPen(pen);
	for(int i = 0; i <= initialDate.daysTo(finalDate); i++)
	{
		//Cambiamos el color de cada semana
		if(initialDate.addDays(i).dayOfWeek() == 1)
		{
			if(pen.color() == QColor(220, 220, 220))
				pen.setColor(QColor(80, 80, 80));
			else
				pen.setColor(QColor(220, 220, 220));
			
			painter.setPen(pen);
		}
		painter.drawText(30 + i * 5 + i * 10 * activeColumns.size(), 42 + graphicHeight,
			initialDate.addDays(i).toString("dd"));
		painter.drawText(30 + i * 5 + i * 10 * activeColumns.size(), 56 + graphicHeight,
			initialDate.addDays(i).toString("MMM"));
		painter.drawText(30 + i * 5 + i * 10 * activeColumns.size(), 70 + graphicHeight,
			initialDate.addDays(i).toString("yyyy"));
	}
	painter.setPen(Qt::NoPen);
}

void Chart::drawGraphics()
{
	calculateTopHour();
	setWidth();
	//update();
}

void Chart::drawLines0(QPainter &painter)
{
	if(!activeColumns.size())
		return;
	
	int totalDays = initialDate.daysTo(finalDate);

	QPen penLines, penDays;
	penLines.setStyle(Qt::SolidLine);
	penLines.setWidth(2);
	penDays.setStyle(Qt::SolidLine);
	penDays.setWidth(1);
	penDays.setColor(QColor(220, 220, 220));
	
	//Dibujamos para cada día su texto bajo la gráfica y la línea vertical correspondiente de la gráfica
	painter.setPen(penDays);
	for(int i = 0; i <= totalDays; i++)
	{
		//Cambiamos el color de cada semana
		if(initialDate.addDays(i).dayOfWeek() == 1)
		{
			if(penDays.color() == QColor(220, 220, 220))
				penDays.setColor(QColor(80, 80, 80));
			else
				penDays.setColor(QColor(220, 220, 220));
			
			painter.setPen(penDays);
		}
		painter.drawText(30 - 6 + i * lineDaySeparation, 42 + graphicHeight,
			initialDate.addDays(i).toString("dd"));
		painter.drawText(30 - 6 + i * lineDaySeparation, 56 + graphicHeight,
			initialDate.addDays(i).toString("MMM"));
		painter.drawText(30 - 6 + i * lineDaySeparation, 70 + graphicHeight,
			initialDate.addDays(i).toString("yyyy"));
		
		//painter.drawLine(30 + i * lineDaySeparation, 25,
		//	30 + i * lineDaySeparation, graphicHeight + 25);
	}
	
	//Dibujamos la gráfica con antialias y luego desactivamos el antialias
	painter.setRenderHints(QPainter::Antialiasing);
	bool first;
	QPointF point1, point2;
	QString queryString;
	QSqlQuery query;
	for(int i = 0; i < userNames.size(); i++)
	{
		for(int j = 0; j < activeColumnsPerUser[i]; j++)
		{
			queryString = "SELECT * FROM Diary" + userNames[i]
				+ " WHERE Date >= '" + initialDate.toString("yyyy-MM-dd")
				+ "' AND Date <= '" + finalDate.toString("yyyy-MM-dd") + "' ORDER BY Date";
			query.exec(qPrintable(queryString));
			first = true;
			
			//Asignamos el color
			penLines.setColor(activeColors[j]);
			painter.setPen(penLines);
			while(query.next())
			{
				//Asignamos la posición del punto del día actual
				point1.setX(30.0 + initialDate.daysTo(query.value(1).toDate()) * lineDaySeparation);
				point1.setY(
					graphicHeight + 25.0
					- qreal(query.value(3 + activeColumns[j]).toTime().hour())
					* (graphicHeight / topHour)
					- qreal(query.value(3 + activeColumns[j]).toTime().minute())
					* (graphicHeight / (topHour * 60.0)));
				
				//Si no es el primer registro, dibujamos la línea con el punto del día anterior y el del actual
				if(first)
					first = false;
				else
					painter.drawLine(point2, point1);
				
				//Asignamos la posición del punto del día actual para usarla en el siguiente registro
				point2.setX(point1.x());
				point2.setY(point1.y());
			}
		}
	}
	painter.setRenderHints(QPainter::Antialiasing, false);
	
	/*
	static int a = 0;
	a++;
	painter.drawText(30, 50, QString::number(a));
	*/
	
	painter.setPen(Qt::NoPen);
}

void Chart::drawLines1(QPainter &painter)
{
	if(!activeColumns.size())
		return;
	
	int totalDays = initialDate.daysTo(finalDate);
	int weekCount = 0;
	int cycleDaysCount = 0;
	
	QPen penLines, penDays;
	penLines.setStyle(Qt::SolidLine);
	penLines.setWidth(2);
	penDays.setStyle(Qt::SolidLine);
	penDays.setWidth(1);
	penDays.setColor(QColor(220, 220, 220));
	painter.setPen(penDays);
	
	//Dibujamos para cada día su texto bajo la gráfica y la línea vertical correspondiente de la gráfica
	for(int i = 0; i <= totalDays; i++)
	{
		if(initialDate.addDays(i).dayOfWeek() == 7 || i == totalDays)
		{
			painter.drawText(30 - 6 + weekCount * lineDaySeparation, 42 + graphicHeight,
				initialDate.addDays(i).toString("dd"));
			painter.drawText(30 - 6 + weekCount * lineDaySeparation, 56 + graphicHeight,
				initialDate.addDays(i).toString("MMM"));
			painter.drawText(30 - 6 + weekCount * lineDaySeparation, 70 + graphicHeight,
				initialDate.addDays(i).toString("yyyy"));
			
			weekCount++;
		}
	}
	
	//Dibujamos la gráfica con antialias y luego desactivamos el antialias
	painter.setRenderHints(QPainter::Antialiasing);
	QPointF point1, point2;
	bool first, last;
	int minuteCount;
	QString queryString;
	QSqlQuery query;
	for(int i = 0; i < userNames.size(); i++)
	{
		for(int j = 0; j < activeColumnsPerUser[i]; j++)
		{
			queryString = "SELECT * FROM Diary" + userNames[i]
				+ " WHERE Date >= '" + initialDate.toString("yyyy-MM-dd")
				+ "' AND Date <= '" + finalDate.toString("yyyy-MM-dd") + "' ORDER BY Date";
			query.exec(qPrintable(queryString));
			if(!query.next())
				break;
				
			first = true;
			last = false;
			weekCount = 0;
			minuteCount = 0;
			cycleDaysCount = 0;
			
			//Asignamos el color
			penLines.setColor(activeColors[j]);
			painter.setPen(penLines);
			for(int k = 0; k < initialDate.daysTo(finalDate) + 1; k++)
			{
				//Añadimos los nuevos valores si hay registro del día correspondiente
				if(last == false && k == initialDate.daysTo(query.value(1).toDate()))
				{
					minuteCount += query.value(3 + activeColumns[j]).toTime().hour() * 60
						+ query.value(3 + activeColumns[j]).toTime().minute();
					cycleDaysCount++;
				}
				
				//Dibujamos el punto anterior y reiniciamos el contador de horas totales al finalizar la semana
				//o al llegar al último día
				if(initialDate.addDays(k).dayOfWeek() == 7 || k == initialDate.daysTo(finalDate))
				{
					//Evitamos que se produzca una división por 0 al establecer point1
					if(cycleDaysCount == 0)
						cycleDaysCount = 1;
						
					point1.setX(30.0 + weekCount * lineDaySeparation);
					if(tempo == 1) //Gráfica del total de horas semanales
						point1.setY(graphicHeight + 25.0 - qreal(minuteCount) / 60.0 * graphicHeight / topHour);
					else //Gráfica de la media de horas semanales
						point1.setY(graphicHeight + 25.0 - qreal(minuteCount)
							/ qreal(cycleDaysCount) / 60.0 * graphicHeight / topHour);
						
					//Si no es el primer registro, dibujamos la línea con el punto del día anterior
					//y el del actual
					if(first)
						first = false;
					else
						painter.drawLine(point2, point1);
					
					//Asignamos la posición del punto del día actual para usarla en el siguiente registro
					point2.setX(point1.x());
					point2.setY(point1.y());
					weekCount++;
					minuteCount = 0;
					cycleDaysCount = 0;
				}
				
				//Leemos el siguiente registro o salimos si no hay más registros
				if(last == false && k == initialDate.daysTo(query.value(1).toDate()))
				{
					if(!query.next())
						last = true;
				}
			}
		}
	}
	painter.setRenderHints(QPainter::Antialiasing, false);
	painter.setPen(Qt::NoPen);
}

void Chart::drawLines2(QPainter &painter)
{
	if(!activeColumns.size())
		return;
	
	int totalDays = initialDate.daysTo(finalDate);
	int monthCount = 0;
	int cycleDaysCount = 0;
	
	QPen penLines, penDays;
	penLines.setStyle(Qt::SolidLine);
	penLines.setWidth(2);
	penDays.setStyle(Qt::SolidLine);
	penDays.setWidth(1);
	penDays.setColor(QColor(220, 220, 220));
	painter.setPen(penDays);
	
	//Dibujamos para cada día su texto bajo la gráfica y la línea vertical correspondiente de la gráfica
	for(int i = 0; i <= totalDays; i++)
	{
		if(initialDate.addDays(i).day() == initialDate.addDays(i).daysInMonth() || i == totalDays)
		{
			painter.drawText(30 - 6 + monthCount * lineDaySeparation, 42 + graphicHeight,
				initialDate.addDays(i).toString("dd"));
			painter.drawText(30 - 6 + monthCount * lineDaySeparation, 56 + graphicHeight,
				initialDate.addDays(i).toString("MMM"));
			painter.drawText(30 - 6 + monthCount * lineDaySeparation, 70 + graphicHeight,
				initialDate.addDays(i).toString("yyyy"));
			
			monthCount++;
		}
	}
	
	//Dibujamos la gráfica con antialias y luego desactivamos el antialias
	painter.setRenderHints(QPainter::Antialiasing);
	QPointF point1, point2;
	bool first, last;
	int minuteCount;
	QString queryString;
	QSqlQuery query;
	for(int i = 0; i < userNames.size(); i++)
	{
		for(int j = 0; j < activeColumnsPerUser[i]; j++)
		{
			queryString = "SELECT * FROM Diary" + userNames[i]
				+ " WHERE Date >= '" + initialDate.toString("yyyy-MM-dd")
				+ "' AND Date <= '" + finalDate.toString("yyyy-MM-dd") + "' ORDER BY Date";
			query.exec(qPrintable(queryString));
			if(!query.next())
				break;
			
			first = true;
			last = false;
			monthCount = 0;
			minuteCount = 0;
			cycleDaysCount = 0;
			
			//Asignamos el color
			penLines.setColor(activeColors[j]);
			painter.setPen(penLines);
			for(int k = 0; k < initialDate.daysTo(finalDate) + 1; k++)
			{
				//Añadimos los nuevos valores si hay registro del día correspondiente
				if(last == false && k == initialDate.daysTo(query.value(1).toDate()))
				{
					minuteCount += query.value(3 + activeColumns[j]).toTime().hour() * 60
						+ query.value(3 + activeColumns[j]).toTime().minute();
					cycleDaysCount++;
				}
				
				//Dibujamos el punto anterior y reiniciamos el contador de horas totales al finalizar el mes
				//o al llegar al último día
				if(initialDate.addDays(k).day() == initialDate.addDays(k).daysInMonth()
					|| k == initialDate.daysTo(finalDate))
				{
					//Evitamos que se produzca una división por 0 al establecer point1
					if(cycleDaysCount == 0)
						cycleDaysCount = 1;
						
					point1.setX(30.0 + monthCount * lineDaySeparation);
					if(tempo == 3) //Gráfica del total de horas mensuales
						point1.setY(graphicHeight + 25.0 - qreal(minuteCount) / 60.0 * graphicHeight / topHour);
					else //Gráfica de la media de horas mensuales
						point1.setY(graphicHeight + 25.0 - qreal(minuteCount)
							/ cycleDaysCount / 60.0 * graphicHeight / topHour);
						
					//Si no es el primer registro, dibujamos la línea con el punto del día anterior
					//y el del actual
					if(first)
						first = false;
					else
						painter.drawLine(point2, point1);
				
					//Asignamos la posición del punto del día actual para usarla en el siguiente registro
					point2.setX(point1.x());
					point2.setY(point1.y());
					monthCount++;
					minuteCount = 0;
					cycleDaysCount = 0;
				}
				
				//Leemos el siguiente registro o salimos si no hay más registros
				if(last == false && k == initialDate.daysTo(query.value(1).toDate()))
				{
					if(!query.next())
						last = true;
				}
			}
		}
	}
	painter.setRenderHints(QPainter::Antialiasing, false);
	painter.setPen(Qt::NoPen);
}

void Chart::drawLines3(QPainter &painter)
{
	if(!activeColumns.size())
		return;
	
	int totalDays = initialDate.daysTo(finalDate);
	int yearCount = 0;
	int cycleDaysCount = 0;
	
	QPen penLines, penDays;
	penLines.setStyle(Qt::SolidLine);
	penLines.setWidth(2);
	penDays.setStyle(Qt::SolidLine);
	penDays.setWidth(1);
	penDays.setColor(QColor(220, 220, 220));
	painter.setPen(penDays);
	
	//Dibujamos para cada día su texto bajo la gráfica y la línea vertical correspondiente de la gráfica
	for(int i = 0; i <= totalDays; i++)
	{
		if(initialDate.addDays(i).dayOfYear() == initialDate.addDays(i).daysInYear() || i == totalDays)
		{
			painter.drawText(30 - 6 + yearCount * lineDaySeparation, 42 + graphicHeight,
				initialDate.addDays(i).toString("dd"));
			painter.drawText(30 - 6 + yearCount * lineDaySeparation, 56 + graphicHeight,
				initialDate.addDays(i).toString("MMM"));
			painter.drawText(30 - 6 + yearCount * lineDaySeparation, 70 + graphicHeight,
				initialDate.addDays(i).toString("yyyy"));
			
			yearCount++;
		}
	}
	
	//Dibujamos la gráfica con antialias y luego desactivamos el antialias
	painter.setRenderHints(QPainter::Antialiasing);
	QPointF point1, point2;
	bool first, last;
	int minuteCount;
	QString queryString;
	QSqlQuery query;
	for(int i = 0; i < userNames.size(); i++)
	{
		for(int j = 0; j < activeColumnsPerUser[i]; j++)
		{
			queryString = "SELECT * FROM Diary" + userNames[i]
				+ " WHERE Date >= '" + initialDate.toString("yyyy-MM-dd")
				+ "' AND Date <= '" + finalDate.toString("yyyy-MM-dd") + "' ORDER BY Date";
			query.exec(qPrintable(queryString));
			if(!query.next())
				break;
			
			first = true;
			last = false;
			yearCount = 0;
			minuteCount = 0;
			cycleDaysCount = 0;
			
			//Asignamos el color
			penLines.setColor(activeColors[j]);
			painter.setPen(penLines);
			for(int k = 0; k < initialDate.daysTo(finalDate) + 1; k++)
			{
				//Añadimos los nuevos valores si hay registro del día correspondiente
				if(last == false && k == initialDate.daysTo(query.value(1).toDate()))
				{
					minuteCount += query.value(3 + activeColumns[j]).toTime().hour() * 60
						+ query.value(3 + activeColumns[j]).toTime().minute();
					cycleDaysCount++;
				}
				
				//Dibujamos el punto anterior y reiniciamos el contador de horas totales al finalizar el mes
				//o al llegar al último día
				if(initialDate.addDays(k).dayOfYear() == initialDate.addDays(k).daysInYear()
					|| k == initialDate.daysTo(finalDate))
				{
					//Evitamos que se produzca una división por 0 al establecer point1
					if(cycleDaysCount == 0)
						cycleDaysCount = 1;
						
					point1.setX(30.0 + yearCount * lineDaySeparation);
					if(tempo == 5) //Gráfica del total de horas anuales
						point1.setY(graphicHeight + 25.0 - qreal(minuteCount) / 60.0 * graphicHeight / topHour);
					else //Gráfica de la media de horas anuales
						point1.setY(graphicHeight + 25.0 - qreal(minuteCount)
							/ cycleDaysCount / 60.0 * graphicHeight / topHour);
						
					//Si no es el primer registro, dibujamos la línea con el punto del día anterior
					//y el del actual
					if(first)
						first = false;
					else
						painter.drawLine(point2, point1);
				
					//Asignamos la posición del punto del día actual para usarla en el siguiente registro
					point2.setX(point1.x());
					point2.setY(point1.y());
					yearCount++;
					minuteCount = 0;
					cycleDaysCount = 0;
				}
				
				//Leemos el siguiente registro o salimos si no hay más registros
				if(last == false && k == initialDate.daysTo(query.value(1).toDate()))
				{
					if(!query.next())
						last = true;
				}
			}
		}
	}
	painter.setRenderHints(QPainter::Antialiasing, false);
	painter.setPen(Qt::NoPen);
}

int Chart::rowCount()
{
	QString queryString;
	QSqlQuery query;
	int rows = 0;
	for(int i = 0; i < userNames.size(); i++)
	{
		//Contamos las columnas de cada usuario solo si tiene columnas activadas
		if(activeColumnsPerUser[i])
		{
			queryString = "SELECT Id FROM Diary" + userNames[i] + " WHERE Date >= '"
				+ initialDate.toString("yyyy-MM-dd") + "' AND Date <= '" + finalDate.toString("yyyy-MM-dd")
				+ "' ORDER BY Date";
			query.exec(qPrintable(queryString));
			while(query.next())
				rows ++;
		}
	}
	return rows;
}

void Chart::setActiveColors(QVector <QColor> &colors)
{
	activeColors = colors;
}

void Chart::setActiveColumns(QVector <int> &columns)
{
	activeColumns = columns;
}

void Chart::setActiveColumnsPerUser(QVector <int> &columns)
{
	activeColumnsPerUser = columns;
	drawGraphics();
}

void Chart::setChartType(int type)
{
	chartType = type;
	drawGraphics();
}

void Chart::setFinalDate(const QDate &newDate)
{
	finalDate = newDate;
	drawGraphics();
}

void Chart::setInitialDate(const QDate &newDate)
{
	initialDate = newDate;
	drawGraphics();
}

void Chart::setTempo(int num)
{
	tempo = num;
	drawGraphics();
}

void Chart::setHeight()
{
	graphicHeight = height() - 80;
}

void Chart::setWidth()
{
	int widgetWidth = 0;
	if(activeColumns.size())
	{
		switch(chartType)
		{
		//Gráfico de barras
		case 0:
			widgetWidth = 30 + (activeColumns.size() * (initialDate.daysTo(finalDate) + 1) * 10)
				+ ((initialDate.daysTo(finalDate) + 1) * 5);
		break;
		//Gráfico de líneas
		case 1:
			switch(tempo)
			{
			case 0:
				widgetWidth = 30 + 30 + initialDate.daysTo(finalDate) * lineDaySeparation;
			break;
			case 1:
			case 2:
				widgetWidth = 30 + 30 + (initialDate.daysTo(finalDate) / 7 + 2) * lineDaySeparation;
			break;
			case 3:
			case 4:
				widgetWidth = 30 + 30 + (initialDate.daysTo(finalDate) / 30 + 2) * lineDaySeparation;
			break;
			case 5:
			case 6:
				widgetWidth = 30 + 30 + (initialDate.daysTo(finalDate) / 365 + 2) * lineDaySeparation;
			break;
			}
		break;
		}
	}
	if(widgetWidth > 0)
		setMinimumSize(widgetWidth, 0);
	
	//Repintamos solo si el nuevo tamaño es menor que el anterior, porque si es mayor ya se ha
	//repintado automáticamente con setMinimumSize() 
	//if(widgetWidth <= width())
	//if(widgetWidth == width())
		update();
}



//PROTECTED


void Chart::paintEvent(QPaintEvent */*event*/)
{
	//setWidth();
	setHeight();
	
	QPainter painter(this);
	painter.save();
	painter.setBrush(QColor(220, 220, 220));
	
	drawBackgroundLines(painter);
	
	switch(chartType)
	{
	case 0:
		drawBars(painter);
	break;
	case 1:
		switch(tempo)
		{
		case 0:
			drawLines0(painter);
		break;
		case 1:
		case 2:
			drawLines1(painter);
		break;
		case 3:
		case 4:
			drawLines2(painter);
		break;
		case 5:
		case 6:
			drawLines3(painter);
		break;
		}
	break;
	}
	
	painter.restore();
}
