/***************************************************************************
 *   Copyright (C) 2008, 2009 by Miguel Escudero Gestal
 *   miguelescudero@users.sourceforge.net
 *   
 *   This file is part of QChartDiary
 *
 *   QChartDiary is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   QChartDiary is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.
 ***************************************************************************/

#ifndef CHARTWINDOWIMPL_H
#define CHARTWINDOWIMPL_H

#include <QMainWindow>
#include <QVector>
#include "chart.h"
#include "dateedit.h"
#include "ui_chartwindow.h"

class QDate;
class QStringList;
class QTreeWidgetItem;


class ChartWindowImpl : public QMainWindow, public Ui::ChartWindow
{
Q_OBJECT
public:
	ChartWindowImpl(QWidget *parent = 0, Qt::WFlags f = 0);
	QDate getFinalDate() {return dateEditFinalDate->date();}
	QDate getInitialDate() {return dateEditInitialDate->date();}

public slots:
	void activateAllUserColumns(QTreeWidgetItem *, int);
	void sendActiveColumns();
	void setComboTempoItems(int);
	void setFinalDate(const QDate &);
	void setInitialDate(const QDate &);
	void updateChart() {chart->update();}
	void writeSettings();

protected:
	void paintEvent(QPaintEvent *event);
	
private:
	void createObjects();
	void createTreeWidgetItems();
	void readSettings();
	
	Chart *chart;
	DateEdit *dateEditInitialDate;
	DateEdit *dateEditFinalDate;
	QStringList userNames;
	QList <QTreeWidgetItem *> treeWidgetItems;
	
signals:
	void userChanged(const QString &name);
};
#endif //CHARTWINDOWIMPL_H
