/***************************************************************************
 *   Copyright (C) 2008, 2009 by Miguel Escudero Gestal
 *   miguelescudero@users.sourceforge.net
 *   
 *   This file is part of QChartDiary
 *
 *   QChartDiary is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   QChartDiary is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.
 ***************************************************************************/

#include <QApplication>
#include <QMessageBox>
#include <QTextCodec>
#include <QTranslator>
#include <QtSql>
#include "mainwindowimpl.h"

bool createConnection()
{
	QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE");
	db.setDatabaseName(QDir::homePath() + "/.config/QChartDiary/diary.db");
	if(!db.open())
	{
		QMessageBox::critical(0, QApplication::tr("Error while trying to open the database"),
			QApplication::tr("This program needs support for SQLite databases."), QMessageBox::Cancel);
		return false;
	}
	return true;
}

void checkDiaryDir()
{
	//Si no existe el directorio en el que se deben guardar las configuraciones, lo creamos
	if(!QDir(QDir::homePath() + "/.config/QChartDiary").exists())
	{
		if(!QDir(QDir::homePath() + "/.config").exists())
			QDir().mkdir(QDir::homePath() + "/.config");
		QDir().mkdir(QDir::homePath() + "/.config/QChartDiary");
	}
	
	//Si no existe el archivo diary.db en el directorio de configuración pero si que existe en el
	//del programa, informamos de que el programa ha sido actualizado y ahora el archivo se encuentra
	//en el directorio de configuración. No funciona si se ejecuta la aplicación directamente desde
	//Konqueror o Dolphin, pues usan un directorio por defecto, en vez del de la aplicación ejecutada
	if(!QFile::exists(QDir::homePath() + "/.config/QChartDiary/diary.db")
		&& QFile::exists(QCoreApplication::applicationDirPath() + "/diary.db"))
	{
		QFile file(QCoreApplication::applicationDirPath() + "/diary.db");
		file.rename(QDir::homePath() + "/.config/QChartDiary/diary.db");
		QMessageBox::information(0, QApplication::tr("Data updated correctly"),
			QApplication::tr("The file containing user introduced data (diary.db) have been correctly moved to ")
			+ QDir::homePath() + "/.config/QChartDiary/diary.db "
			+ QApplication::tr("That means that you've previously used a version of QChartDiary prior to 0.6.3.1 "
			"and it was necesary to move this file to use the new version of the program keeping the old data."));
	}
}

int main(int argc, char ** argv)
{
	QApplication app(argc, argv);

	//Establecemos los datos para QSettings
	QCoreApplication::setOrganizationName("QChartDiary");
	QCoreApplication::setOrganizationDomain("qchartdiary.sourceforge.net");
	QCoreApplication::setApplicationName("QChartDiary");
	
	//Leemos la configuración para establecer el idioma de la aplicación
	QSettings settings;
	settings.beginGroup("General");
	QTranslator translator;
	if(!translator.load("QChartDiary_" + settings.value("language", QLocale::system().name()).toString()))
	{
		if(!translator.load(QCoreApplication::applicationDirPath() + "/QChartDiary_"
			+ settings.value("language", QLocale::system().name()).toString()))
		{
			translator.load("/usr/share/QChartDiary/QChartDiary_"
				+ settings.value("language", QLocale::system().name()).toString());
		}
	}
	app.installTranslator(&translator);
	settings.endGroup();
	
	//Configuramos el codec que usará la función tr()
	//QTextCodec::setCodecForTr(QTextCodec::codecForName("UTF-8"));
	
	checkDiaryDir();
	
	//Creación de la conexión a la base de datos
	if(!createConnection())
		return 1;
	
	MainWindowImpl mainWindow;
	mainWindow.show();
	
	//El cierre de la aplicación ya queda implementado luego
	//app.connect(&app, SIGNAL(lastWindowClosed()), &app, SLOT(quit()));
	app.setQuitOnLastWindowClosed(false);
	return app.exec();
}
