/***************************************************************************
 *   Copyright (C) 2008, 2009 by Miguel Escudero Gestal
 *   miguelescudero@users.sourceforge.net
 *   
 *   This file is part of QChartDiary
 *
 *   QChartDiary is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   QChartDiary is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.
 ***************************************************************************/
 
#include "mainwindowimpl.h"

#include <QCloseEvent>
#include <QColorDialog>
#include <QFileDialog>
#include <QInputDialog>
//#include <QLocale>
#include <QSettings>
#include <QSqlQuery>
#include <QTextStream>

//#include <iostream>
//using namespace std;


//PUBLIC


MainWindowImpl::MainWindowImpl(QWidget * parent, Qt::WFlags f) 
	: QMainWindow(parent, f)
{
	setupUi(this);
	
	readSettings();
	
	//Establecemos la localización española
	//locale = new QLocale(QLocale::Spanish, QLocale::Spain);
	//setLocale(*locale);
	
	//Inicializamos algunas variables
	close = false;
	dataChanged = false;
	chartWindow = 0;
	tasksWindow = 0;
	textViewWindow = 0;
	setWindowTitle("QChartDiary 0.6 - " + userName + " [*]");
	setWindowModified(false);
	
	createObjects();
	//createSystemTrayIcon();
	
	currentDate = configureDate();
	openRegister(currentDate);
	
	//En algún sitio dice que cuando una signal se conecta a varios slots, no se sigue ningún orden particular
	//de ejecución, pero el funcionamiento de estas dos parece desmentirlo
	connect(actionQuit, SIGNAL(triggered()), this, SLOT(setClose()));
	connect(actionQuit, SIGNAL(triggered()), this, SLOT(close()));

	//connect(systemTrayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
	//	this, SLOT(systemTrayIconActivated(QSystemTrayIcon::ActivationReason)));
	
	connect(actionAbout, SIGNAL(triggered()), this, SLOT(about()));
	connect(actionCancelChanges, SIGNAL(triggered()), this, SLOT(cancelChanges()));
	connect(actionChangeUser, SIGNAL(triggered()), this, SLOT(changeUser()));
	connect(actionCreateUser, SIGNAL(triggered()), this, SLOT(createUser()));
	connect(actionDeleteUser, SIGNAL(triggered()), this, SLOT(deleteUser()));
	connect(actionModifyUserName, SIGNAL(triggered()), this, SLOT(modifyUserName()));
	connect(actionSelectLanguage, SIGNAL(triggered()), this, SLOT(selectLanguage()));
	connect(actionShowChartWindow, SIGNAL(triggered()), this, SLOT(showChartWindow()));
	connect(actionShowTasksWindow, SIGNAL(triggered()), this, SLOT(showTasksWindow()));
	connect(actionShowTasksWindowOnStartup, SIGNAL(triggered(bool)), this, SLOT(setShowTasksWindowOnStartup(bool)));
	connect(actionShowTextViewWindow, SIGNAL(triggered()), this, SLOT(showTextViewWindow()));
	connect(actionSelectFutureDaysColor, SIGNAL(triggered()), this, SLOT(selectFutureDaysColor()));
	connect(actionSelectPastDaysColor, SIGNAL(triggered()), this, SLOT(selectPastDaysColor()));
	connect(btnCancelChanges, SIGNAL(clicked()), this, SLOT(cancelChanges()));
	connect(btnChangeUser, SIGNAL(clicked()), this, SLOT(changeUser()));
	connect(btnShowChartWindow, SIGNAL(clicked()), this, SLOT(showChartWindow()));
	connect(btnShowTasksWindow, SIGNAL(clicked()), this, SLOT(showTasksWindow()));
	connect(btnShowTextViewWindow, SIGNAL(clicked()), this, SLOT(showTextViewWindow()));
	connect(calendarWidget, SIGNAL(selectionChanged()), this, SLOT(changeDay()));
	connect(textEdit, SIGNAL(textChanged()), this, SLOT(setModified()));
	
	//Column options
	connect(btnDeleteColumn, SIGNAL(clicked()), this, SLOT(deleteHourCol()));
	connect(btnInsertColumn, SIGNAL(clicked()), this, SLOT(insertHourCol()));
	connect(btnModifyColumn, SIGNAL(clicked()), this, SLOT(modifyHourCol()));
	
	showInfo(tr("Welcome!"));
	
	if(showTasksWindowOnStartup == true)
		showTasksWindow();
		
	setCursor(QTextCursor::End);
}


//PUBLIC SLOTS


void MainWindowImpl::about()
{
	QMessageBox::about(this, tr("About..."), tr("QChartDiary was created by Miguel Escudero "
		"<miguelescudero@users.sourceforge.net>. The web page of the application is "
		"http://qchartdiary.sourceforge.net/. If you need more info, read the README file located "
		"in the main directory of the application."));
}

void MainWindowImpl::alterTableDiary(const QString &columnNames, const QString &columnNamesAndTypes,
	const QString &newColumnNames, const QString &newColumnNamesAndTypes)
{
	QSqlQuery queryDiary;
	
	//Field elimination
	if(newColumnNames.isEmpty() && newColumnNamesAndTypes.isEmpty())
	{
		queryDiary.exec("CREATE TEMPORARY TABLE Diary" + userName + "Backup ("
			+ columnNamesAndTypes + ");");
		queryDiary.exec("INSERT INTO Diary" + userName + "Backup SELECT "
			+ columnNames + " FROM Diary" + userName + ";");
		queryDiary.exec("DROP TABLE Diary" + userName + ";");
		queryDiary.exec("CREATE TABLE Diary" + userName + " (" + columnNamesAndTypes + ");");
		queryDiary.exec("INSERT INTO Diary" + userName + " (" + columnNames + ") SELECT "
			+ columnNames + " FROM Diary" + userName + "Backup;");
	}
	
	//Field modification
	else
	{
		queryDiary.exec("CREATE TEMPORARY TABLE Diary" + userName + "Backup ("
			+ newColumnNamesAndTypes + ");");
		queryDiary.exec("INSERT INTO Diary" + userName + "Backup SELECT "
			+ columnNames + " FROM Diary" + userName + ";");
		queryDiary.exec("DROP TABLE Diary" + userName + ";");
		queryDiary.exec("CREATE TABLE Diary" + userName + " (" + newColumnNamesAndTypes + ");");
		queryDiary.exec("INSERT INTO Diary" + userName + " SELECT "
			+ newColumnNames + " FROM Diary" + userName + "Backup;");
	}

	queryDiary.exec("DROP TABLE Diary" + userName + "Backup;");
}

QMessageBox::StandardButton MainWindowImpl::askQuestion(const QString &title, const QString &question)
{
	QMessageBox messageBox;
	messageBox.setIcon(QMessageBox::Question);
	messageBox.setWindowTitle(title);
	messageBox.setText(question);
	messageBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
	messageBox.setDefaultButton(QMessageBox::No);
	return QMessageBox::StandardButton(messageBox.exec());
}

QString MainWindowImpl::arrangeString(QString stringAux)
{
	QString validCharacters("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789");
	int length = stringAux.length();
	
	if(stringAux[0].isLower())
		stringAux[0] = stringAux[0].toUpper();
	for(int i = 0; i < length; i++)
	{
		//Ponemos en mayúsculas cualquier letra que aparezca tras un espacio, y eliminamos el espacio
		if(stringAux[i].isSpace())
		{
			if(stringAux[i + 1].isLower())
			{
				stringAux[i + 1] = stringAux[i + 1].toUpper(); 
			}
			stringAux = stringAux.mid(0, i) + stringAux.mid(i + 1, stringAux.length() - (i + 1));
			i--;
			length = stringAux.length();
		}
		//Eliminamos los caracteres que no sean letras o dígitos
		else if(validCharacters.indexOf(stringAux[i]) == -1)
		{
			stringAux = stringAux.mid(0, i) + stringAux.mid(i + 1, stringAux.length() - (i + 1));
			i--;
			length = stringAux.length();
		}
	}
	return stringAux;
}

void MainWindowImpl::backup()
{
	//Habría que realizar una copia del archivo de la base de datos
}

void MainWindowImpl::cancelChanges()
{
	openRegister(currentDate);
	showInfo(tr("Changes made to the last register were cancelled"));
}

void MainWindowImpl::changeDay()
{
	saveRegister(currentDate);
	currentDate=configureDate();
	openRegister(currentDate);
}

void MainWindowImpl::changeFutureDaysColor()
{
	//Si existen anotaciones de texto en algún día posterior al actual,
	//se cambiará el color del fondo de ese día
	QTextCharFormat format;
	format.setBackground(futureDaysColor);
	QSqlQuery query;
	query.exec("SELECT Date, FullText FROM Diary" + userName + " ORDER BY Date");
	while(query.next())
	{
		if(!query.value(1).toString().isEmpty() && query.value(0).toDate().daysTo(QDate::currentDate()) < 0)
		{
			calendarWidget->setDateTextFormat(query.value(0).toDate(), format);
		}
		
		//Devolvemos a ese día el color original
		else if(query.value(1).toString().isEmpty())
		{
			calendarWidget->setDateTextFormat(query.value(0).toDate(),
				calendarWidget->weekdayTextFormat(Qt::Tuesday));
		}
	}
}

void MainWindowImpl::changePastDaysColor()
{
	//Si existen anotaciones de texto en algún día anterior al actual o en el actual,
	//se cambiará el color del fondo de ese día
	QTextCharFormat format;
	format.setBackground(pastDaysColor);
	QSqlQuery query;
	query.exec("SELECT Date, FullText FROM Diary" + userName + " ORDER BY Date");
	while(query.next() && query.value(0).toDate().daysTo(QDate::currentDate()) >= 0)
	{
		if(!query.value(1).toString().isEmpty())
		{
			calendarWidget->setDateTextFormat(query.value(0).toDate(), format);
		}
		
		//Devolvemos a ese día el color original
		else
		{
			calendarWidget->setDateTextFormat(query.value(0).toDate(),
				calendarWidget->weekdayTextFormat(Qt::Tuesday));
		}
	}
}

void MainWindowImpl::changeUser(QString name, bool save)
{
	if(save)
		saveRegister(currentDate);
	
	//Si no se ha especificado el nombre de usuario, mostramos un diálogo de selección con los usuarios disponibles
	if(name.isEmpty())
	{
		name = userNameSelectionDialog(tr("Select a user from the list"), true);
		if(name.isEmpty())
			return;
	}
	
	resetDaysColor();
	userName = name;
	destroyHourWidgets();
	createHourWidgets();
	openRegister(currentDate);
	setWindowTitle("QChartDiary 0.6 - " + userName + " [*]");
	
	emit userChanged(userName);
}

QString MainWindowImpl::columnNameSelectionDialog(const QString &messageString, bool selectItem)
{
	bool ok = 0;
	QString columnName;
	
	//Si selectItem == true, mostraremos un dialogo de selección con una lista de los campos disponibles
	if(selectItem)
	{
		QStringList columnNames;
		QSqlQuery query;
		query.exec("SELECT FullName FROM Fields" + userName);
		for(int i = 0; query.next(); i++)
		{
			columnNames<<query.value(0).toString();
		}
		columnName = QInputDialog::getItem(this, tr("Field selection"),
			messageString, columnNames, 0, false, &ok);
	}
	
	//Si selectItem == false, pedimos que se introduzca un nombre
	else
	{
		columnName = QInputDialog::getText(this, tr("Intro"), messageString, QLineEdit::Normal, "", &ok);
	}
	
	if(!ok)
		return "";
	if(columnName.isEmpty() || columnName == "Id" || columnName == "Date" || columnName == "FullText")
	{
		QMessageBox::information(this, "Info", "El nombre introducido es incorrecto");
		return "";
	}
	return columnName;
}

int MainWindowImpl::countRowsInDiary()
{
	//Algoritmo para conocer el Id del último registro
	QSqlQuery queryRowCount;
	queryRowCount.exec("SELECT * FROM Diary" + userName);
	queryRowCount.last();
	return queryRowCount.value(0).toInt();
}

QString MainWindowImpl::configureDate()
{
	QString date = calendarWidget->selectedDate().toString("yyyy-MM-dd");
	return date;
}

void MainWindowImpl::createTables(QString name)
{
	if(name.isEmpty())
		name = userName;
		
	QSqlQuery query;
	table->setTable("Diary" + name);
	if(!table->select())
	{
		query.exec("CREATE TABLE Diary" + name + "("
			"Id INTEGER PRIMARY KEY, "
			"Date TEXT, "
			"FullText TEXT)");
		showInfo(tr("Diary table created successfully."), true);
	}
	table->setTable("Fields" + name);
	if(!table->select())
	{
		query.exec("CREATE TABLE Fields" + name + "("
			"Id INTEGER PRIMARY KEY, "
			"FieldNumberInDiary TEXT, "
			"Name TEXT, "
			"FullName TEXT)");
		showInfo(tr("Fields table created successfully."), true);
	}
	
	table->setTable("Diary" + name);
}

void MainWindowImpl::createUser()
{
	//Mostramos el diálogo de introducción de nombre de usuario
	QString name = userNameSelectionDialog(tr("Enter a new user name "
		"(Note that it must start with a letter and only letters and numbers can be introduced)"));
	if(name.isEmpty())
		return;
	
	//Si el nombre ya se encontraba en la BD, avisamos de que no es válido y cancelamos la creación
	if(findUserName(name))
	{
		QMessageBox::information(this, tr("Create user"), tr("The selected name is already registered, "
			"try again with an unregistered name"));
		return;
	}
	
	//Comprobamos que el nombre sea válido, hacemos los ajustes necesarios al mismo, mostramos el nombre ajustado
	//y le preguntamos al usuario si está conforme con el nombre reformateado
	QString nameAux = arrangeString(name);
	QString question(tr("To validate the name, it was reformatted. The resultant name is ")
		+ nameAux + tr(". Are you satisfied with the new name?"));
	if(askQuestion(tr("New user creation"), question) == QMessageBox::Yes)
		name = nameAux;
	else
	{
		QMessageBox::warning(this, tr("Error"), tr("Creation cancelled by the user"));
		return;
	}
	
	//Creamos las dos nuevas tablas para el usuario en la BD
	createTables(name);
	
	//Preguntamos si se desea cambiar al nuevo usuario
	if(askQuestion(tr("New user creation"),
		tr("New user created. Do you want to use the new user's account right now?")) == QMessageBox::Yes)
	{
		changeUser(name);
	}
}

void MainWindowImpl::deleteHourCol()
{
	QString columnName = columnNameSelectionDialog(tr("Select the name of the field that you want to delete"), true);
	if(columnName.isEmpty())
		return;
	
	int columnId = findHourCol(columnName);
	if(columnId)
	{
		//Borramos el registro en la tabla FieldsUserName
		table->setTable("Fields" + userName);
		table->select();
		table->removeRow(columnId - 1);
		table->submitAll();
		table->select();
		
		//Cambiamos los campos Id y FieldNumberInDiary de todos los registros que siguen al eliminado
		for(int row = columnId - 1; row < table->rowCount(); row++)
		{
			table->setData(table->index(row, 0), row + 1);
			table->setData(table->index(row, 1), QString::number(table->record(row).value(1).toInt() - 1));
		}
		table->submitAll();
		
		//Creamos una cadena con los nombres y otra con los nombres y tipos de los campos
		QString columnNames = "Id, Date, FullText, ";
		QString columnNamesAndTypes = "Id INTEGER PRIMARY KEY, Date TEXT, FullText TEXT, ";
		table->setTable("Fields" + userName);
		table->select();
		for(int row = 0; row < table->rowCount(); row++)
		{
			columnNames += table->record(row).value(2).toString() + ", ";
			columnNamesAndTypes += table->record(row).value(2).toString() + " TEXT, ";
		}
		
		//Eliminamos la última coma y el último espacio
		columnNames = columnNames.left(columnNames.length() - 2);
		columnNamesAndTypes = columnNamesAndTypes.left(columnNamesAndTypes.length() - 2);
		
		//Borramos el campo en la tabla DiaryUserName
		alterTableDiary(columnNames, columnNamesAndTypes);
		
		//Eliminamos el hourWidget correspondiente
		delete hourWidgets[columnId - 1];
		hourWidgets.remove(columnId - 1);
		
		showInfo(tr("Field deleted sucesfully"));
		table->setTable("Diary" + userName);
		table->select();
	}
	else
	{
		QMessageBox::information(this, tr("Info"), tr("The field does not exist"));
	}
}

void MainWindowImpl::deleteUser()
{
	//Mostramos el diálogo de selección de usuario
	QString name = userNameSelectionDialog(tr("Select the user that you want to delete "
		"(Do not choose the actual user, because I can't delete a user that's being used)"), true);
	if(name.isEmpty())
		return;
		
	//Si se está intentando eliminar el usuario actual, se advertirá de que no se puede
	if(name == userName)
	{
		QMessageBox::warning(this, tr("Error"), tr("Sorry, I can't delete the user that is in use "
			"right now. You must change to other user and then you will be able to delete this"));
		return;
	}
	
	//Preguntamos si está seguro de que desea eliminar el usuario, advirtiendo que los cambios serán irrevocables
	QString question(tr("Are you absolutely sure that you want to delete the selected user and all his data?"));
	if(askQuestion(tr("User elimination"), question) == QMessageBox::Yes)
	{
		//Eliminamos las dos tablas de la BD
		QSqlQuery query;
		query.exec("DROP TABLE IF EXISTS Diary" + name);
		query.exec("DROP TABLE IF EXISTS Fields" + name);
		
		//Informamos de que el usuario y sus datos han sido eliminados definitivamente
		QMessageBox::information(this, tr("User elimination"), tr("The user ") + name
			+ tr(" and all his data have been eliminated"));
	}
}

void MainWindowImpl::destroyHourWidgets()
{
	for(int i = 0; i < hourWidgets.size(); i++)
		delete hourWidgets[i];
	hourWidgets.clear();
}

int MainWindowImpl::findHourCol(const QString &columnName)
{
	table->setTable("Fields" + userName);
	QSqlQuery query;
	query.exec("SELECT Id, FullName FROM Fields" + userName +
		" WHERE FullName = '" + columnName + "'");
	if(query.next())
		return query.value(0).toInt();
	else
		return 0;
}

bool MainWindowImpl::findUserName(const QString &userNameAux)
{
	QSqlQuery query;
	query.exec("SELECT name, sql FROM sqlite_master WHERE type='table'"
		"AND name LIKE 'Diary"+ userNameAux + "' ORDER BY name;");
	
	if(query.next())
		return true;
	else
		return false;
}

void MainWindowImpl::insertHourCol()
{
	//Introducimos el nombre para el nuevo campo
	QString columnName = columnNameSelectionDialog(tr("Enter the name of the new field "
		"(try not to make it very long)"));
	if(columnName.isEmpty())
		return;
	QString columnNameArranged = arrangeString(columnName);
	
	//Controlamos que no se introduzca un campo ya existente
	if(findHourCol(columnName))
	{
		QMessageBox::information(this, tr("Info"),
			tr("The field you tried to enter already exists and cannot be created again"));
	}
	else
	{
		//Contamos el número de campos de la tabla Diary
		table->setTable("Diary" + userName);
		table->select();
		int col = table->columnCount();
		
		//Introducimos el nuevo campo en la tabla Fields
		//Sería convieniente utilizar un QSqlQuery en vez de la tabla, que fallará con más de 256 registros
		table->setTable("Fields" + userName);
		table->select();
		int row = table->rowCount();
		table->insertRow(row);
		table->setData(table->index(row, 0), table->rowCount());
		table->setData(table->index(row, 1), col);
		table->setData(table->index(row, 2), columnNameArranged);
		table->setData(table->index(row, 3), columnName);
		table->submitAll();
		
		//Añadimos el nuevo campo a la tabla Diary y especificamos que será de tipo text
		//y ponemos la hora 00:00:00 en todos los registros para el nuevo campo
		QSqlQuery queryDiary;
		queryDiary.exec("ALTER TABLE Diary" + userName + " ADD " + columnNameArranged + " TEXT");
		queryDiary.exec("UPDATE Diary" + userName + " SET " + columnNameArranged + "='00:00:00'");
			//+ " WHERE Id=" + QString::number(i));
		
		//Creamos el hourWidget correspondiente
		hourWidgets.append(new HourWidget(columnName, this));
		connect(hourWidgets.last(), SIGNAL(timeChanged(const QTime &)), this, SLOT(setModified()));
		verticalLayout->addWidget(hourWidgets.last());
	
		showInfo(tr("Field sucesfully entered"));
		table->setTable("Diary" + userName);
		table->select();
	}
}

void MainWindowImpl::modifyHourCol()
{
	QString columnName = columnNameSelectionDialog(tr("Select the name of the field that you want to modify"), true);
	if(columnName.isEmpty())
		return;
	
	int columnId = findHourCol(columnName);
	if(columnId)
	{
		//Introducimos el nuevo nombre para el campo
		QString newColumnName;
		newColumnName = columnNameSelectionDialog(tr("Enter the new name for the field"));
		if(newColumnName.isEmpty())
			return;
			
		//Comprobamos que el nuevo nombre no sea igual a alguno existente en la tabla
		if(findHourCol(newColumnName))
		{
			QMessageBox::information(this, tr("Info"),
				tr("The field you tried to enter already exists and cannot be created again"));
			return;
		}
		
		//Creamos una cadena con los nombres y otra con los nombres y tipos de los campos antiguos
		table->setTable("Fields" + userName);
		table->select();
		QString columnNames = "Id, Date, FullText";
		QString columnNamesAndTypes = "Id INTEGER PRIMARY KEY, Date TEXT, FullText TEXT";
		for(int row = 0; row < table->rowCount(); row++)
		{
			columnNames += ", " + table->record(row).value(2).toString();
			columnNamesAndTypes += ", " + table->record(row).value(2).toString() + " TEXT";
		}

		//Modificamos el registro en la tabla Fields
		table->setTable("Fields" + userName);
		table->select();
		table->setData(table->index(columnId - 1, 2), arrangeString(newColumnName));
		table->setData(table->index(columnId - 1, 3), newColumnName);
		table->submitAll();
		
		//Creamos una cadena con los nombres y otra con los nombres y tipos de los campos nuevos
		table->setTable("Fields" + userName);
		table->select();
		QString newColumnNames = "Id, Date, FullText";
		QString newColumnNamesAndTypes = "Id INTEGER PRIMARY KEY, Date TEXT, FullText TEXT";
		for(int row = 0; row < table->rowCount(); row++)
		{
			newColumnNames += ", " + table->record(row).value(2).toString();
			newColumnNamesAndTypes += ", " + table->record(row).value(2).toString() + " TEXT";
		}
		
		//Modificamos el campo en la tabla DiaryUserName
		alterTableDiary(columnNames, columnNamesAndTypes, newColumnNames, newColumnNamesAndTypes);
		
		//Modificamos el hourWidget correspondiente
		hourWidgets[columnId - 1]->setText(newColumnName);
		
		showInfo(tr("Field modified sucesfully"));
		table->setTable("Diary" + userName);
		table->select();
	}
	else
	{
		QMessageBox::information(this, tr("Info"), tr("The field does not exist"));
	}
}

void MainWindowImpl::modifyUserName()
{
	//Mostramos el diálogo de selección de usuario
	QString name = userNameSelectionDialog(tr("Select the user name that you want to modify"), true);
	if(name.isEmpty())
		return;
	
	//Mostramos el diálogo de introducción del nuevo nombre de usuario
	QString newName = userNameSelectionDialog(tr("Enter a new user name "
		"(Note that it must start with a letter and only letters and numbers can be introduced)"));
	if(newName.isEmpty())
		return;
	
	//Si el nombre ya se encontraba en la BD, avisamos de que no es válido y cancelamos la modificación
	if(findUserName(newName))
	{
		QMessageBox::information(this, tr("User modification"), tr("The selected name is already "
			"registered, try again with an unregistered name"));
		return;
	}
	
	//Comprobamos que el nombre sea válido, hacemos los ajustes necesarios al mismo, mostramos el nombre ajustado
	//y le preguntamos al usuario si está conforme con el nombre reformateado
	QString nameAux = arrangeString(newName);
	QString question(tr("To validate the name, it was reformatted. The resultant name is ")
		+ nameAux + tr(". Are you satisfied with the new name?"));
	if(askQuestion(tr("User modification"), question) == QMessageBox::Yes)
		name = nameAux;
	else
	{
		QMessageBox::warning(this, tr("Error"), tr("Modification cancelled by the user"));
		return;
	}
	
	//Modificamos la BD
	QSqlQuery query;
	query.exec("ALTER TABLE Diary" + name + " RENAME TO Diary" + newName);
	query.exec("ALTER TABLE Fields" + name + " RENAME TO Fields" + newName);
	
	//Si el nombre que se ha cambiado es el del usuario actual, hacemos el cambio de usuario correspondiente
	if(name == userName)
		changeUser(newName, false);
}

void MainWindowImpl::openRegister(const QString &date)
{
	//Cambiamos el color de los días con anotaciones
	changePastDaysColor();
	changeFutureDaysColor();
	
	QString stringDiary;
	stringDiary = "SELECT * FROM Diary" + userName + " WHERE Date IN ('" + date + "')";
	QSqlQuery queryDiary;
	queryDiary.exec(stringDiary);

	QString stringFields;
	stringFields = "SELECT Id, FieldNumberInDiary, Name FROM Fields" + userName;
	QSqlQuery queryFields;
	queryFields.exec(stringFields);
	
	//Si no hay registro del día seleccionado, borramos el contenido de los widgets y no abrimos nada
	if(!queryDiary.next())
	{
		textEdit->clear();
		while(queryFields.next())
		{
			QTime initialTime(0,0);
			hourWidgets[queryFields.value(0).toInt() - 1]->setTime(initialTime);
		}
		setCursor(QTextCursor::End);
		setModified(false);
		return;
	}
	
	//Si falta alguna hora, la rellenamos con el valor 00:00:00
	QString modifications;
	bool changes = false;
	queryFields.exec(stringFields);
	while(queryFields.next())
	{
		if(queryDiary.value(queryFields.value(1).toInt()) == "")
		{
			modifications += queryFields.value(2).toString() + "='00:00:00', ";
			changes = true;
		}
	}
	
	//Si ha habido cambios, actualizamos el registro utilizando la cadena recien creada
	if(changes == true)
	{
		modifications = modifications.left(modifications.length() - 2);
		queryDiary.exec("UPDATE Diary" + userName + " SET " + modifications
			+ " WHERE Id=" + queryDiary.value(0).toString());
		
		showInfo(tr("Data from the day ") + date + tr(" saved."));
	}
	
	//Rellenamos los widgets con la información del registro
	queryDiary.exec(stringDiary);
	queryDiary.next();
	textEdit->setText(queryDiary.value(2).toString());
	queryFields.exec(stringFields);
	while(queryFields.next())
	{
		hourWidgets[queryFields.value(0).toInt() - 1]->
			setTime(queryDiary.value(queryFields.value(1).toInt()).toTime());
	}
	
	setCursor(QTextCursor::End);
	setModified(false);
}

void MainWindowImpl::readSettings()
{
	QSettings settings;
	
	settings.beginGroup("MainWindow");
	//restoreGeometry(settings.value("geometry").toByteArray());
	//resize(settings.value("size", QSize(800, 655)).toSize());
	//move(settings.value("pos", QPoint(100, 100)).toPoint());
	setGeometry(settings.value("geometry", QRect(100, 100, 800, 655)).toRect());
	userName = settings.value("userName", "User").toString();
	futureDaysColor = settings.value("futureDaysColor", QColor(255, 255, 20)).value<QColor>();
	pastDaysColor = settings.value("pastDaysColor", QColor(88, 143, 225)).value<QColor>();
	showTasksWindowOnStartup = settings.value("showTasksWindowOnStartup", false).toBool();
	actionShowTasksWindowOnStartup->setChecked(settings.value("showTasksWindowOnStartup", false).toBool());
	settings.endGroup();
	
	settings.beginGroup("General");
	language = settings.value("language", "en").toString();
	settings.endGroup();
}

void MainWindowImpl::resetDaysColor()
{
	//Reiniciamos el color de todos los días anteriormente coloreados
	QSqlQuery query;
	query.exec("SELECT Date, FullText FROM Diary" + userName + " ORDER BY Date");
	while(query.next())
	{
		calendarWidget->setDateTextFormat(query.value(0).toDate(), 
			calendarWidget->weekdayTextFormat(Qt::DayOfWeek(query.value(0).toDate().dayOfWeek())));
	}
}

void MainWindowImpl::restoreDefaultSettings()
{
	QSettings settings;
	settings.clear();
}

void MainWindowImpl::saveRegister(const QString &date)
{
	//Grabaremos el registro del día actual en la base de datos
	if(!dataChanged)
		return;
	
	QString rows = QString::number(countRowsInDiary());
	
	QString stringDiary;
	stringDiary = "SELECT * FROM Diary" + userName + " WHERE Date IN ('" + date + "')";
	QSqlQuery queryDiary;
	queryDiary.exec(stringDiary);

	QString stringFields;
	stringFields = "SELECT Id, Name FROM Fields" + userName;
	QSqlQuery queryFields;
	queryFields.exec(stringFields);
	
	//Creamos un nuevo registro si no existe, e incluímos Id y Date
	if(!queryDiary.next())
	{
		queryDiary.exec("INSERT INTO Diary" + userName + " (Id, Date) "
			"VALUES (" + QString::number(rows.toInt()+1) + ", '" + date + "')");
		
		//Volvemos a realizar la consulta una vez añadido el nuevo registro, porque sinó luego daría error
		queryDiary.exec(stringDiary);
		queryDiary.next();
	}
	
	//Creamos una cadena que almacene todos los nombres de campos y sus nuevos valores
	//Cambiamos todas las comillas simples por comillas dobles en el texto, pues sinó no se puede
	//introducir correctamente en la base de datos
	QString modifications = "FullText='" + textEdit->document()->toPlainText().replace("'", "''") + "'";
	while(queryFields.next())
	{
		modifications += ", " + queryFields.value(1).toString() + "='"
			+ hourWidgets[queryFields.value(0).toInt() - 1]->time().toString() + "'";
	}
	
	//Actualizamos el registro utilizando la cadena recien creada
	queryDiary.exec("UPDATE Diary" + userName + " SET " + modifications
		+ " WHERE Id=" + queryDiary.value(0).toString());

	//Informamos de que el registro se ha guardado
	showInfo(tr("Data from the day ") + date + tr(" saved."));
	emit registerSaved();
}

void MainWindowImpl::selectFutureDaysColor()
{
	QColor color;
	color = QColorDialog::getColor(futureDaysColor, this);
	if(!color.isValid())
		return;
	futureDaysColor = color;
	changeFutureDaysColor();
}

void MainWindowImpl::selectLanguage()
{
	bool ok;
	QStringList availableLanguages;
	availableLanguages<<tr("en (english)")<<tr("es (spanish)")<<tr("gl (galician)");
	availableLanguages<<tr("ja (japanese) (not available yet)");
	availableLanguages<<tr("zhCN (simplified chinese) (not available yet)");
	
	QString selectedLanguage = QInputDialog::getItem(this, tr("Language selection"),
		tr("Select application language (you must restart before changes will take effect)"),
		availableLanguages, 0, false, &ok);
	
	if(ok && !selectedLanguage.isEmpty())
	{
		language = selectedLanguage.left(2);
		QMessageBox::information(this, tr("Language selection"),
			tr("You must restart the application before changes will take effect"));
	}
}

void MainWindowImpl::selectPastDaysColor()
{
	QColor color;
	color = QColorDialog::getColor(pastDaysColor, this);
	if(!color.isValid())
		return;
	pastDaysColor = color;
	changePastDaysColor();
}

void MainWindowImpl::setClose()
{
	close = true;
}

void MainWindowImpl::setCursor(QTextCursor::MoveOperation operation)
{
	textEdit->setFocus(); //¿Realmente hace falta?
	textCursor->movePosition(operation, QTextCursor::MoveAnchor, 1);
	textEdit->setTextCursor(*textCursor);
}

void MainWindowImpl::setModified(bool value)
{
	setWindowModified(value);
	actionCancelChanges->setEnabled(value);
	btnCancelChanges->setEnabled(value);
	dataChanged = value;
}

void MainWindowImpl::showChartWindow()
{
	if(!chartWindow)
	{
		chartWindow = new ChartWindowImpl(this);
		
		//Con esto conseguimos que la ventana de gráficas no aparezca en la barra de tareas
		chartWindow->setWindowFlags(chartWindow->windowFlags() | Qt::Dialog);
		connect(this, SIGNAL(registerSaved()), chartWindow, SLOT(updateChart()));
	}
	if(chartWindow->isHidden())
	{
		chartWindow->show();
	}
	else
	{
		chartWindow->activateWindow();
	}
}

void MainWindowImpl::showInfo(const QString &info, /*bool console, */int seconds)
{
	statusBar()->showMessage(info, seconds * 1000);
	//if(console == true)
	//	cerr<<endl<<qPrintable(informacion)<<endl;
}

void MainWindowImpl::showTasksWindow()
{
	if(!tasksWindow)
	{
		tasksWindow = new TasksWindowImpl(userName, this);
		connect(this, SIGNAL(userChanged(const QString &)), tasksWindow, SLOT(setUser(const QString &)));
		//emit userChanged(userName);
	}
	if(tasksWindow->isHidden())
	{
		tasksWindow->show();
	}
	else
	{
		tasksWindow->activateWindow();
	}
}

void MainWindowImpl::showTextViewWindow()
{
	if(!textViewWindow)
	{
		textViewWindow = new TextViewWindowImpl(userName, this);
		connect(this, SIGNAL(userChanged(const QString &)), textViewWindow, SLOT(setUser(const QString &)));
	}
	if(textViewWindow->isHidden())
	{
		textViewWindow->show();
	}
	else
	{
		textViewWindow->activateWindow();
	}
}

void MainWindowImpl::systemTrayIconActivated(QSystemTrayIcon::ActivationReason reason)
{
	switch(reason)
	{
		case QSystemTrayIcon::Unknown:
		break;
		case QSystemTrayIcon::Context:
		break;
		case QSystemTrayIcon::DoubleClick:
		break;
		case QSystemTrayIcon::Trigger:
			if(this->isVisible())
				hide();
			else
				show();
		break;
		case QSystemTrayIcon::MiddleClick:
		break;
	}
}

QString MainWindowImpl::userNameSelectionDialog(const QString &messageString, bool selectItem)
{
	bool ok = 0;
	QString userNameAux;
	
	//Si selectItem == true, mostraremos un dialogo de selección con una lista de los nombres de usuario disponibles
	if(selectItem)
	{
		QStringList userNamesAux;
		QSqlQuery query;
		query.exec("SELECT name, sql FROM sqlite_master WHERE type='table'"
		"AND name LIKE 'Diary%' ORDER BY name;");
		for(int i = 0; query.next(); i++)
		{
			userNamesAux<<query.value(0).toString().mid(5);
		}
		userNameAux = QInputDialog::getItem(this, tr("Select a user name"),
			messageString, userNamesAux, 0, false, &ok);
	}
	
	//Si selectItem == false, pedimos que se introduzca un nombre de usuario
	else
	{
		userNameAux = QInputDialog::getText(this, tr("Intro"), messageString, QLineEdit::Normal, "", &ok);
	}
	
	if(!ok)
		return "";
	if(userNameAux.isEmpty())
	{
		QMessageBox::information(this, tr("Info"), tr("Incorrect name entered"));
		return "";
	}
	return userNameAux;
}

void MainWindowImpl::writeSettings()
{
	QSettings settings;
	
	settings.beginGroup("MainWindow");
	//settings.setValue("geometry", saveGeometry());
	//settings.setValue("size", size());
	//settings.setValue("pos", pos());
	settings.setValue("geometry", geometry());
	settings.setValue("userName", userName);
	settings.setValue("futureDaysColor", futureDaysColor);
	settings.setValue("pastDaysColor", pastDaysColor);
	settings.setValue("showTasksWindowOnStartup", showTasksWindowOnStartup);
	settings.endGroup();
	
	settings.beginGroup("General");
	settings.setValue("language", language);
	settings.endGroup();
}



//PROTECTED


void MainWindowImpl::closeEvent(QCloseEvent *event)
{
	/*if(close == false && systemTrayIcon->isVisible())
	{
		hide();
		event->ignore();
	}
	else*/
	{
		saveRegister(currentDate);
		
		if(actionRestoreDefaultSettings->isChecked())
		{
			restoreDefaultSettings();
		}
		else
		{
			writeSettings();
			if(chartWindow)
				chartWindow->writeSettings();
		}
		qApp->quit();
		event->accept();
	}
}


//PRIVATE


void MainWindowImpl::createHourWidgets()
{
	verticalLayout->setAlignment(Qt::AlignTop);
	
	//Creamos los hourWidgets con la información correspondiente que se encuentra en la BD
	QSqlQuery query;
	query.exec("SELECT Id, FullName FROM Fields" + userName);
	while(query.next())
	{
		hourWidgets.append(new HourWidget(query.value(1).toString(), this));
		connect(hourWidgets.last(), SIGNAL(timeChanged(const QTime &)),
			this, SLOT(setModified()));
		verticalLayout->addWidget(hourWidgets.last());
	}
}

void MainWindowImpl::createObjects()
{
	table = new QSqlTableModel(this);
	table->setEditStrategy(QSqlTableModel::OnManualSubmit);
	createTables();
	
	createHourWidgets();
	textCursor = new QTextCursor(textEdit->textCursor());
}

void MainWindowImpl::createSystemTrayIcon()
{
	actionShowMainWindow = new QAction(tr("&Show main window"), this);
	connect(actionShowMainWindow, SIGNAL(triggered()), this, SLOT(show()));
	
	actionHideMainWindow = new QAction(tr("&Hide main window"), this);
	connect(actionHideMainWindow, SIGNAL(triggered()), this, SLOT(hide()));
	
	trayIcon = new QIcon("./images/systrayicon.png");
	
	trayMenu = new QMenu(tr("Menu"), this);
	trayMenu->addAction(actionAbout);
	trayMenu->addSeparator();
	trayMenu->addAction(actionShowMainWindow);
	trayMenu->addAction(actionHideMainWindow);
	trayMenu->addSeparator();
	trayMenu->addAction(actionQuit);
	
	systemTrayIcon = new QSystemTrayIcon(*trayIcon, this);
	systemTrayIcon->setContextMenu(trayMenu);
	systemTrayIcon->show();
}
