/***************************************************************************
 *   Copyright (C) 2008, 2009 by Miguel Escudero Gestal
 *   miguelescudero@users.sourceforge.net
 *   
 *   This file is part of Agenda
 *
 *   Agenda is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Agenda is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.
 ***************************************************************************/

#ifndef MAINWINDOWIMPL_H
#define MAINWINDOWIMPL_H

#include <QMainWindow>
#include <QMessageBox>
#include <QtSql>
#include <QSystemTrayIcon>
#include "chartwindowimpl.h"
#include "hourwidget.h"
#include "taskswindowimpl.h"
#include "textviewwindowimpl.h"
#include "ui_mainwindow.h"

//Forward declarations
class QAction;
class QCloseEvent;
class QIcon;
//class QLocale;
class QMenu;
class QTextCursor;


class MainWindowImpl : public QMainWindow, public Ui::MainWindow
{
Q_OBJECT
public:
	MainWindowImpl(QWidget * parent = 0, Qt::WFlags f = 0);

public slots:
	void about();
	void alterTableDiary(const QString &, const QString &,
		const QString &newColumnNames = "", const QString &newColumnNamesAndTypes = "");
	QString arrangeString(QString);
	QMessageBox::StandardButton askQuestion(const QString &, const QString &);
	void backup();
	void cancelChanges();
	void changeDay();
	void changeFutureDaysColor();
	void changePastDaysColor();
	void changeUser(QString name = "", bool save = true);
	QString columnNameSelectionDialog(const QString &, bool selectItem = false);
	int countRowsInDiary();
	QString configureDate();
	void createTables(QString name = "");
	void createUser();
	void deleteHourCol();
	void deleteUser();
	void destroyHourWidgets();
	int findHourCol(const QString &);
	bool findUserName(const QString &);
	void insertHourCol();
	void modifyHourCol();
	void modifyUserName();
	void openRegister(const QString &);
	void readSettings();
	void resetDaysColor();
	void restoreDefaultSettings();
	void saveRegister(const QString &);
	void selectFutureDaysColor();
	void selectLanguage();
	void selectPastDaysColor();
	void setClose();
	void setCursor(QTextCursor::MoveOperation);
	void setModified(bool value = true);
	void setShowTasksWindowOnStartup(bool value) {showTasksWindowOnStartup = value;}
	void showChartWindow();
	void showInfo(const QString &, /*bool console = false, */int seconds = 4);
	void showTasksWindow();
	void showTextViewWindow();
	void systemTrayIconActivated(QSystemTrayIcon::ActivationReason);
	QString userNameSelectionDialog(const QString &, bool selectItem = false);
	void writeSettings();

protected:
	void closeEvent(QCloseEvent *event);
	
private:
	void createHourWidgets();
	void createObjects();
	void createSystemTrayIcon();
	
	QAction *actionHideMainWindow;
	QAction *actionShowMainWindow;
	bool close;
	bool dataChanged;
	bool showTasksWindowOnStartup;
	//QLocale *locale;
	QString language;
	QString currentDate;
	QString userName;
	QSystemTrayIcon *systemTrayIcon;
	QSqlTableModel *table;
	QTextCursor *textCursor;
	QIcon *trayIcon;
	QMenu *trayMenu;
	QVector <HourWidget *> hourWidgets;
	QColor futureDaysColor;
	QColor pastDaysColor;
	ChartWindowImpl *chartWindow;
	TasksWindowImpl *tasksWindow;
	TextViewWindowImpl *textViewWindow;
	
signals:
	void userChanged(const QString &name);
	void registerSaved();
};
#endif //MAINWINDOWIMPL_H
