/***************************************************************************
 *   Copyright (C) 2008, 2009 by Miguel Escudero Gestal
 *   miguelescudero@users.sourceforge.net
 *   
 *   This file is part of QChartDiary
 *
 *   QChartDiary is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   QChartDiary is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.
 ***************************************************************************/

#include "textviewwindowimpl.h"
#include <QMessageBox>
#include <QSqlQuery>


TextViewWindowImpl::TextViewWindowImpl(const QString &name, QWidget * parent, Qt::WFlags f)
	: QDialog(parent, f)
{
	setupUi(this);

	userName = name;
	textCursor = new QTextCursor(textEdit->textCursor());
	
	connect(btnSearch, SIGNAL(clicked()), this, SLOT(searchText()));
	connect(checkCase, SIGNAL(stateChanged(int)), this, SLOT(setCase(int)));

	showText();
}

void TextViewWindowImpl::setUser(const QString &name)
{
	userName = name;
	textEdit->clear();
	showText();
}

void TextViewWindowImpl::setCase(int state)
{
	if(state == 0)
	{
		findFlags = 0;
	}
	else if(state == 2)
	{
		findFlags = QTextDocument::FindCaseSensitively;
	}
}

void TextViewWindowImpl::searchText()
{
	//Si el cursor ha llegado al final preguntamos si se desea volver a empezar
	if(!textEdit->find(lineEdit->text(), findFlags))
	{
		QMessageBox msgBox;
		msgBox.setText(tr("The selected text wasn't found"));
		msgBox.setInformativeText(tr("Would you like to search again from the beginning?"));
		msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
		msgBox.setDefaultButton(QMessageBox::Yes);
		switch(msgBox.exec())
		{
		case QMessageBox::Yes:
			setCursor(QTextCursor::Start);
			searchText();
		break;
		case QMessageBox::No:
		break;
		}
	}
}

void TextViewWindowImpl::setCursor(QTextCursor::MoveOperation operation)
{
	textCursor->movePosition(operation, QTextCursor::MoveAnchor, 1);
	textEdit->setTextCursor(*textCursor);
}

void TextViewWindowImpl::showText()
{
	//Busca solo en los registros comprendidos entre las fechas especificadas. En realidad quizás sea mejor quitar
	//lo de las fechas, pues para buscar es mejor que esté todo el texto. Quizás estaría bien limitar la selección
	//a años o meses, en vez de a días.
	//Dale formato al texto. Puedes cambiar el color de fondo para cada mes, poner la fecha subrayada, etc.
	QSqlQuery query;
	query.exec("SELECT Date, FullText FROM Diary" + userName + " WHERE FullText <> '' ORDER BY Date");
	while(query.next())
	{
		textEdit->textCursor().insertText(query.value(0).toString() + "\n"
			+ query.value(1).toString() + "\n\n");
		//textEdit->textCursor().insertHtml("<h3><u>" + query.value(0).toString() + "</u></h3>" 
		//	+ "<pre>" + query.value(1).toString() + "</pre>" + "<p></p>");
	}
	
	setCursor(QTextCursor::Start);
	lineEdit->setFocus();
}
